@@ -1,11 +1,9 @@
-/* Copyright 2013--2016 The Tor Project
+/* Copyright 2013--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.onionoo.docs;
 
 import org.torproject.onionoo.util.FormattingUtils;
-import org.torproject.onionoo.util.Time;
-import org.torproject.onionoo.util.TimeFactory;
 
 import com.google.gson.Gson;
 import com.google.gson.GsonBuilder;
@@ -52,12 +50,6 @@ public class DocumentStore {
     this.outDir = outDir;
   }
 
-  private Time time;
-
-  public DocumentStore() {
-    this.time = TimeFactory.getTime();
-  }
-
   private long listOperations = 0L;
 
   private long listedFiles = 0L;
@@ -115,15 +107,14 @@ public class DocumentStore {
       this.cacheNodeStatuses();
     }
     if (updatedAfter >= this.lastModifiedNodeStatuses) {
-      return new TreeSet<String>(this.updatedNodeStatuses);
+      return new TreeSet<>(this.updatedNodeStatuses);
     } else {
-      return new TreeSet<String>(this.cachedNodeStatuses.keySet());
+      return new TreeSet<>(this.cachedNodeStatuses.keySet());
     }
   }
 
   private void cacheNodeStatuses() {
-    SortedMap<String, NodeStatus> parsedNodeStatuses =
-        new TreeMap<String, NodeStatus>();
+    SortedMap<String, NodeStatus> parsedNodeStatuses = new TreeMap<>();
     File directory = this.statusDir;
     if (directory != null) {
       File summaryFile = new File(directory, "summary");
@@ -150,7 +141,7 @@ public class DocumentStore {
       }
     }
     this.cachedNodeStatuses = parsedNodeStatuses;
-    this.updatedNodeStatuses = new TreeSet<String>();
+    this.updatedNodeStatuses = new TreeSet<>();
   }
 
   private SortedSet<String> listSummaryDocuments(long updatedAfter) {
@@ -158,15 +149,14 @@ public class DocumentStore {
       this.cacheSummaryDocuments();
     }
     if (updatedAfter >= this.lastModifiedSummaryDocuments) {
-      return new TreeSet<String>(this.updatedSummaryDocuments);
+      return new TreeSet<>(this.updatedSummaryDocuments);
     } else {
-      return new TreeSet<String>(this.cachedSummaryDocuments.keySet());
+      return new TreeSet<>(this.cachedSummaryDocuments.keySet());
     }
   }
 
   private void cacheSummaryDocuments() {
-    SortedMap<String, SummaryDocument> parsedSummaryDocuments =
-        new TreeMap<String, SummaryDocument>();
+    SortedMap<String, SummaryDocument> parsedSummaryDocuments = new TreeMap<>();
     if (this.outDir != null) {
       File summaryFile = new File(this.outDir, "summary");
       if (summaryFile.exists()) {
@@ -198,12 +188,12 @@ public class DocumentStore {
       }
     }
     this.cachedSummaryDocuments = parsedSummaryDocuments;
-    this.updatedSummaryDocuments = new TreeSet<String>();
+    this.updatedSummaryDocuments = new TreeSet<>();
   }
 
   private <T extends Document> SortedSet<String> listDocumentFiles(
       Class<T> documentType, long updatedAfter) {
-    SortedSet<String> fingerprints = new TreeSet<String>();
+    SortedSet<String> fingerprints = new TreeSet<>();
     File directory = null;
     String subdirectory = null;
     if (documentType.equals(DetailsStatus.class)) {
@@ -238,7 +228,7 @@ public class DocumentStore {
       subdirectory = "uptimes";
     }
     if (directory != null && subdirectory != null) {
-      Stack<File> files = new Stack<File>();
+      Stack<File> files = new Stack<>();
       files.add(new File(directory, subdirectory));
       while (!files.isEmpty()) {
         File file = files.pop();
@@ -418,7 +408,7 @@ public class DocumentStore {
     boolean isRelay = detailsDocument.getHashedFingerprint() == null;
     boolean running = false;
     String nickname = detailsDocument.getNickname();
-    List<String> addresses = new ArrayList<String>();
+    List<String> addresses = new ArrayList<>();
     String countryCode = null;
     String asNumber = null;
     String contact = null;
@@ -443,15 +433,16 @@ public class DocumentStore {
         }
       }
     }
-    SortedSet<String> relayFlags = new TreeSet<String>();
+    SortedSet<String> relayFlags = new TreeSet<>();
     SortedSet<String> family = null;
+    String version = null;
     long lastSeenMillis = -1L;
     long consensusWeight = -1L;
     long firstSeenMillis = -1L;
     SummaryDocument summaryDocument = new SummaryDocument(isRelay,
         nickname, fingerprint, addresses, lastSeenMillis, running,
         relayFlags, consensusWeight, countryCode, firstSeenMillis,
-        asNumber, contact, family, family);
+        asNumber, contact, family, family, version);
     return summaryDocument;
   }
 
@@ -547,8 +538,9 @@ public class DocumentStore {
     Gson gson = new Gson();
     try {
       result = gson.fromJson(documentString, documentType);
-    } catch (JsonParseException e) {
+    } catch (Throwable e) {
       /* Handle below. */
+      log.error(documentString);
       log.error(e.getMessage(), e);
     }
     if (result == null) {
@@ -727,10 +719,8 @@ public class DocumentStore {
       return;
     }
     File summaryFile = new File(directory, "summary");
-    SortedMap<String, NodeStatus> cachedRelays =
-        new TreeMap<String, NodeStatus>();
-    SortedMap<String, NodeStatus> cachedBridges =
-        new TreeMap<String, NodeStatus>();
+    SortedMap<String, NodeStatus> cachedRelays = new TreeMap<>();
+    SortedMap<String, NodeStatus> cachedBridges = new TreeMap<>();
     for (Map.Entry<String, NodeStatus> e :
         this.cachedNodeStatuses.entrySet()) {
       if (e.getValue().isRelay()) {
@@ -818,7 +808,7 @@ public class DocumentStore {
       return;
     }
     UpdateStatus updateStatus = new UpdateStatus();
-    updateStatus.setUpdatedMillis(this.time.currentTimeMillis());
+    updateStatus.setUpdatedMillis(System.currentTimeMillis());
     this.store(updateStatus);
   }
 
