@@ -1,4 +1,4 @@
-/* Copyright 2011--2016 The Tor Project
+/* Copyright 2011--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.onionoo.docs;
@@ -76,7 +76,7 @@ public class NodeStatus extends Document {
   }
 
   private SortedSet<String> stringArrayToSortedSet(String[] stringArray) {
-    SortedSet<String> sortedSet = new TreeSet<String>();
+    SortedSet<String> sortedSet = new TreeSet<>();
     if (stringArray != null) {
       sortedSet.addAll(Arrays.asList(stringArray));
     }
@@ -137,7 +137,7 @@ public class NodeStatus extends Document {
    * includes the primary address and all additionally configured
    * onion-routing addresses. */
   public SortedSet<String> getOrAddresses() {
-    SortedSet<String> orAddresses = new TreeSet<String>();
+    SortedSet<String> orAddresses = new TreeSet<>();
     if (this.address != null) {
       orAddresses.add(this.address);
     }
@@ -193,11 +193,9 @@ public class NodeStatus extends Document {
     return this.dirPort;
   }
 
-  private static Map<String, Integer> relayFlagIndexes =
-      new HashMap<String, Integer>();
+  private static Map<String, Integer> relayFlagIndexes = new HashMap<>();
 
-  private static Map<Integer, String> relayFlagStrings =
-      new HashMap<Integer, String>();
+  private static Map<Integer, String> relayFlagStrings = new HashMap<>();
 
   private BitSet relayFlags;
 
@@ -226,7 +224,7 @@ public class NodeStatus extends Document {
 
   @SuppressWarnings("checkstyle:javadocmethod")
   public SortedSet<String> getRelayFlags() {
-    SortedSet<String> result = new TreeSet<String>();
+    SortedSet<String> result = new TreeSet<>();
     if (this.relayFlags != null) {
       for (int i = this.relayFlags.nextSetBit(0); i >= 0;
           i = this.relayFlags.nextSetBit(i + 1)) {
@@ -267,10 +265,10 @@ public class NodeStatus extends Document {
   }
 
   private SortedMap<Long, Set<String>> lastAddresses =
-      new TreeMap<Long, Set<String>>(Collections.reverseOrder());
+      new TreeMap<>(Collections.reverseOrder());
 
   public SortedMap<Long, Set<String>> getLastAddresses() {
-    return new TreeMap<Long, Set<String>>(this.lastAddresses);
+    return new TreeMap<>(this.lastAddresses);
   }
 
   /** Adds addresses and ports together with the time in milliseconds
@@ -278,7 +276,7 @@ public class NodeStatus extends Document {
    * addresses and ports. */
   public void addLastAddresses(long lastSeenMillis, String address,
       int orPort, int dirPort, SortedSet<String> orAddressesAndPorts) {
-    Set<String> addressesAndPorts = new HashSet<String>();
+    Set<String> addressesAndPorts = new HashSet<>();
     addressesAndPorts.add(address + ":" + orPort);
     if (dirPort > 0) {
       addressesAndPorts.add(address + ":" + dirPort);
@@ -323,6 +321,17 @@ public class NodeStatus extends Document {
     return this.recommendedVersion;
   }
 
+  private String version;
+
+  public void setVersion(String version) {
+    this.version = null == version ? null
+        : version.substring(version.lastIndexOf(" ") + 1);
+  }
+
+  public String getVersion() {
+    return this.version;
+  }
+
   /* From exit lists: */
 
   private SortedSet<String> exitAddresses;
@@ -332,7 +341,7 @@ public class NodeStatus extends Document {
   }
 
   public SortedSet<String> getExitAddresses() {
-    return new TreeSet<String>(this.exitAddresses);
+    return new TreeSet<>(this.exitAddresses);
   }
 
   /* GeoIP lookup results: */
@@ -396,7 +405,7 @@ public class NodeStatus extends Document {
    * declared family that are not in a mutual family relationship with
    * this relay. */
   public SortedSet<String> getAllegedFamily() {
-    SortedSet<String> allegedFamily = new TreeSet<String>(
+    SortedSet<String> allegedFamily = new TreeSet<>(
         stringArrayToSortedSet(this.declaredFamily));
     allegedFamily.removeAll(stringArrayToSortedSet(this.effectiveFamily));
     return allegedFamily;
@@ -406,7 +415,7 @@ public class NodeStatus extends Document {
    * reached via mutual family relationships except for those that can be
    * reached directly via such a relationship. */
   public SortedSet<String> getIndirectFamily() {
-    SortedSet<String> indirectFamily = new TreeSet<String>(
+    SortedSet<String> indirectFamily = new TreeSet<>(
         stringArrayToSortedSet(this.extendedFamily));
     indirectFamily.removeAll(stringArrayToSortedSet(this.effectiveFamily));
     return indirectFamily;
@@ -433,8 +442,8 @@ public class NodeStatus extends Document {
       NodeStatus nodeStatus = new NodeStatus(fingerprint);
       nodeStatus.setRelay(parts[0].equals("r"));
       nodeStatus.setNickname(parts[1]);
-      SortedSet<String> orAddressesAndPorts = new TreeSet<String>();
-      SortedSet<String> exitAddresses = new TreeSet<String>();
+      SortedSet<String> orAddressesAndPorts = new TreeSet<>();
+      SortedSet<String> exitAddresses = new TreeSet<>();
       String addresses = parts[3];
       String address = null;
       if (addresses.contains(";")) {
@@ -465,13 +474,17 @@ public class NodeStatus extends Document {
         log.error("Parse exception while parsing node status "
             + "line '" + documentString + "'.  Skipping.");
         return null;
+      } else if (lastSeenMillis == 0L) {
+        log.debug("Skipping node status with fingerprint {} that has so far "
+            + "never been seen in a network status.", fingerprint);
+        return null;
       }
       nodeStatus.setLastSeenMillis(lastSeenMillis);
       int orPort = Integer.parseInt(parts[6]);
       int dirPort = Integer.parseInt(parts[7]);
       nodeStatus.setOrPort(orPort);
       nodeStatus.setDirPort(dirPort);
-      nodeStatus.setRelayFlags(new TreeSet<String>(
+      nodeStatus.setRelayFlags(new TreeSet<>(
           Arrays.asList(parts[8].split(","))));
       nodeStatus.setConsensusWeight(Long.parseLong(parts[9]));
       nodeStatus.setCountryCode(parts[10]);
@@ -523,9 +536,9 @@ public class NodeStatus extends Document {
          *  2. indirect members that can be reached via others only.
          * Each group contains zero or more ';'-separated fingerprints. */
         String[] groups = parts[22].split(":", -1);
-        SortedSet<String> allegedFamily = new TreeSet<String>();
-        SortedSet<String> effectiveFamily = new TreeSet<String>();
-        SortedSet<String> indirectFamily = new TreeSet<String>();
+        SortedSet<String> allegedFamily = new TreeSet<>();
+        SortedSet<String> effectiveFamily = new TreeSet<>();
+        SortedSet<String> indirectFamily = new TreeSet<>();
         if (groups[0].length() > 0) {
           allegedFamily.addAll(Arrays.asList(groups[0].split(";")));
         }
@@ -535,16 +548,19 @@ public class NodeStatus extends Document {
         if (groups.length > 2 && groups[2].length() > 0) {
           indirectFamily.addAll(Arrays.asList(groups[2].split(";")));
         }
-        SortedSet<String> declaredFamily = new TreeSet<String>();
+        SortedSet<String> declaredFamily = new TreeSet<>();
         declaredFamily.addAll(allegedFamily);
         declaredFamily.addAll(effectiveFamily);
         nodeStatus.setDeclaredFamily(declaredFamily);
         nodeStatus.setEffectiveFamily(effectiveFamily);
-        SortedSet<String> extendedFamily = new TreeSet<String>();
+        SortedSet<String> extendedFamily = new TreeSet<>();
         extendedFamily.addAll(effectiveFamily);
         extendedFamily.addAll(indirectFamily);
         nodeStatus.setExtendedFamily(extendedFamily);
       }
+      if (parts.length >= 24 && !parts[23].isEmpty()) {
+        nodeStatus.setVersion(parts[23]);
+      }
       return nodeStatus;
     } catch (NumberFormatException e) {
       log.error("Number format exception while parsing node "
@@ -565,9 +581,9 @@ public class NodeStatus extends Document {
   public String toString() {
     StringBuilder sb = new StringBuilder();
     sb.append(this.isRelay ? "r" : "b");
-    sb.append("\t" + this.nickname);
-    sb.append("\t" + this.fingerprint);
-    sb.append("\t" + this.address + ";");
+    sb.append("\t").append(this.nickname);
+    sb.append("\t").append(this.fingerprint);
+    sb.append("\t").append(this.address).append(";");
     if (this.orAddressesAndPorts != null) {
       sb.append(StringUtils.join(this.orAddressesAndPorts, "+"));
     }
@@ -575,39 +591,42 @@ public class NodeStatus extends Document {
     if (this.isRelay) {
       sb.append(StringUtils.join(this.exitAddresses, "+"));
     }
-    sb.append("\t" + DateTimeHelper.format(this.lastSeenMillis,
+    sb.append("\t").append(DateTimeHelper.format(this.lastSeenMillis,
         DateTimeHelper.ISO_DATETIME_TAB_FORMAT));
-    sb.append("\t" + this.orPort);
-    sb.append("\t" + this.dirPort + "\t");
+    sb.append("\t").append(this.orPort);
+    sb.append("\t").append(this.dirPort).append("\t");
     sb.append(StringUtils.join(this.getRelayFlags(), ","));
     if (this.isRelay) {
-      sb.append("\t" + String.valueOf(this.consensusWeight));
-      sb.append("\t"
-          + (this.countryCode != null ? this.countryCode : "??"));
+      sb.append("\t").append(String.valueOf(this.consensusWeight));
+      sb.append("\t")
+          .append((this.countryCode != null ? this.countryCode : "??"));
       sb.append("\t"); /* formerly used for storing host names */
-      sb.append("\t" + String.valueOf(this.lastRdnsLookup));
-      sb.append("\t" + (this.defaultPolicy != null ? this.defaultPolicy
-          : "null"));
-      sb.append("\t" + (this.portList != null ? this.portList : "null"));
+      sb.append("\t").append(String.valueOf(this.lastRdnsLookup));
+      sb.append("\t").append((this.defaultPolicy != null
+          ? this.defaultPolicy : "null"));
+      sb.append("\t").append((this.portList != null ? this.portList : "null"));
     } else {
       sb.append("\t-1\t??\t\t-1\tnull\tnull");
     }
-    sb.append("\t" + DateTimeHelper.format(this.firstSeenMillis,
+    sb.append("\t").append(DateTimeHelper.format(this.firstSeenMillis,
         DateTimeHelper.ISO_DATETIME_TAB_FORMAT));
     if (this.isRelay) {
-      sb.append("\t" + DateTimeHelper.format(
+      sb.append("\t").append(DateTimeHelper.format(
           this.getLastChangedOrAddressOrPort(),
           DateTimeHelper.ISO_DATETIME_TAB_FORMAT));
-      sb.append("\t" + (this.asNumber != null ? this.asNumber : "null"));
+      sb.append("\t").append((this.asNumber != null ? this.asNumber : "null"));
     } else {
       sb.append("\tnull\tnull\tnull");
     }
-    sb.append("\t" + (this.contact != null ? this.contact : ""));
-    sb.append("\t" + (this.recommendedVersion == null ? "null" :
+    sb.append("\t").append((this.contact != null ? this.contact : ""));
+    sb.append("\t").append((this.recommendedVersion == null ? "null" :
         this.recommendedVersion ? "true" : "false"));
-    sb.append("\t" + StringUtils.join(this.getAllegedFamily(), ";") + ":"
-        + StringUtils.join(this.getEffectiveFamily(), ";") + ":"
-        + StringUtils.join(this.getIndirectFamily(), ";"));
+    sb.append("\t").append(StringUtils.join(this.getAllegedFamily(), ";"))
+        .append(":")
+        .append(StringUtils.join(this.getEffectiveFamily(), ";")).append(":")
+        .append(StringUtils.join(this.getIndirectFamily(), ";"));
+    sb.append("\t")
+        .append((this.getVersion() != null ? this.getVersion() : ""));
     return sb.toString();
   }
 }
