@@ -3,9 +3,6 @@
 
 package org.torproject.onionoo.server;
 
-import org.torproject.onionoo.util.Time;
-import org.torproject.onionoo.util.TimeFactory;
-
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.ArrayList;
@@ -69,18 +66,30 @@ public class ResourceServlet extends HttpServlet {
   private static Set<String> knownParameters = new HashSet<>(
       Arrays.asList(("type,running,search,lookup,fingerprint,country,as,"
           + "flag,first_seen_days,last_seen_days,contact,order,limit,"
-          + "offset,fields,family").split(",")));
+          + "offset,fields,family,version").split(",")));
 
   private static Set<String> illegalSearchQualifiers =
       new HashSet<>(Arrays.asList(("search,fingerprint,order,limit,"
           + "offset,fields").split(",")));
 
+  private static String ipv6AddressPatternString =
+      "^\\[?[0-9a-fA-F:\\.]{1,39}\\]?$";
+
+  private static Pattern ipv6AddressPattern =
+      Pattern.compile(ipv6AddressPatternString);
+
+  public void doGet(HttpServletRequestWrapper request,
+      HttpServletResponseWrapper response) throws IOException {
+    doGet(request, response, System.currentTimeMillis());
+  }
+
   /** Handles the HTTP GET request in the wrapped <code>request</code> by
    * writing an HTTP GET response to the likewise <code>response</code>,
    * both of which are wrapped to facilitate testing. */
   @SuppressWarnings("checkstyle:variabledeclarationusagedistance")
   public void doGet(HttpServletRequestWrapper request,
-      HttpServletResponseWrapper response) throws IOException {
+      HttpServletResponseWrapper response, long receivedRequestMillis)
+      throws IOException {
 
     if (this.maintenanceMode) {
       response.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
@@ -93,10 +102,6 @@ public class ResourceServlet extends HttpServlet {
       response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
       return;
     }
-
-    Time time = TimeFactory.getTime();
-    long receivedRequestMillis = time.currentTimeMillis();
-
     String uri = request.getRequestURI();
     if (uri.startsWith("/onionoo/")) {
       uri = uri.substring("/onionoo".length());
@@ -150,13 +155,17 @@ public class ResourceServlet extends HttpServlet {
       }
       List<String> unqualifiedSearchTerms = new ArrayList<>();
       for (String searchTerm : searchTerms) {
-        if (searchTerm.contains(":") && !searchTerm.startsWith("[")) {
+        if (searchTerm.contains(":")) {
           String[] parts = searchTerm.split(":", 2);
           String parameterKey = parts[0];
           if (!knownParameters.contains(parameterKey)
               || illegalSearchQualifiers.contains(parameterKey)) {
-            response.sendError(HttpServletResponse.SC_BAD_REQUEST);
-            return;
+            if (ipv6AddressPattern.matcher(parameterKey).matches()) {
+              unqualifiedSearchTerms.add(searchTerm);
+            } else {
+              response.sendError(HttpServletResponse.SC_BAD_REQUEST);
+              return;
+            }
           }
           if (!parameterMap.containsKey(parameterKey)) {
             String parameterValue = parts[1];
@@ -266,6 +275,15 @@ public class ResourceServlet extends HttpServlet {
       }
       rh.setContact(contactParts);
     }
+    if (parameterMap.containsKey("version")) {
+      String versionParameter = this.parseVersionParameter(
+          parameterMap.get("version"));
+      if (null == versionParameter) {
+        response.sendError(HttpServletResponse.SC_BAD_REQUEST);
+        return;
+      }
+      rh.setVersion(versionParameter);
+    }
     if (parameterMap.containsKey("order")) {
       String[] order = this.parseOrderParameter(parameterMap.get("order"));
       if (order == null) {
@@ -313,7 +331,7 @@ public class ResourceServlet extends HttpServlet {
       rh.setFamily(family);
     }
     rh.handleRequest();
-    long parsedRequestMillis = time.currentTimeMillis();
+    long parsedRequestMillis = System.currentTimeMillis();
 
     ResponseBuilder rb = new ResponseBuilder();
     rb.setResourceType(resourceType);
@@ -354,7 +372,7 @@ public class ResourceServlet extends HttpServlet {
     int relayDocumentsWritten = rh.getOrderedRelays().size();
     int bridgeDocumentsWritten = rh.getOrderedBridges().size();
     int charsWritten = rb.getCharsWritten();
-    long writtenResponseMillis = time.currentTimeMillis();
+    long writtenResponseMillis = System.currentTimeMillis();
     PerformanceMetrics.logStatistics(receivedRequestMillis, resourceType,
         parameterMap.keySet(), parsedRequestMillis, relayDocumentsWritten,
         bridgeDocumentsWritten, charsWritten, writtenResponseMillis);
@@ -369,7 +387,7 @@ public class ResourceServlet extends HttpServlet {
       Pattern.compile("^\\$?[0-9a-fA-F]{1,40}$|" /* Hex fingerprint. */
       + "^[0-9a-zA-Z+/]{1,27}$|" /* Base64 fingerprint. */
       + "^[0-9a-zA-Z\\.]{1,19}$|" /* Nickname or IPv4 address. */
-      + "^\\[[0-9a-fA-F:\\.]{1,39}\\]?$|" /* IPv6 address. */
+      + ipv6AddressPatternString + "|" /* IPv6 address. */
       + "^[a-zA-Z_]+:\\p{Graph}+$" /* Qualified search term. */);
 
   protected static String[] parseSearchParameters(String queryString) {
@@ -523,5 +541,16 @@ public class ResourceServlet extends HttpServlet {
     }
     return parameter.toLowerCase().split(",");
   }
+
+  private static Pattern versionParameterPattern =
+      Pattern.compile("^[0-9a-zA-Z\\.-]+$");
+
+  private String parseVersionParameter(String parameter) {
+    if (!versionParameterPattern.matcher(parameter).matches()) {
+      /* Version contains illegal character(s). */
+      return null;
+    }
+    return parameter;
+  }
 }
 
