@@ -1,4 +1,4 @@
-/* Copyright 2011--2016 The Tor Project
+/* Copyright 2011--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.onionoo.updater;
@@ -16,7 +16,6 @@ import org.torproject.onionoo.docs.DocumentStore;
 import org.torproject.onionoo.docs.DocumentStoreFactory;
 import org.torproject.onionoo.docs.NodeStatus;
 import org.torproject.onionoo.util.FormattingUtils;
-import org.torproject.onionoo.util.TimeFactory;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -82,8 +81,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
   private long now;
 
-  private SortedMap<String, NodeStatus> knownNodes =
-      new TreeMap<String, NodeStatus>();
+  private SortedMap<String, NodeStatus> knownNodes = new TreeMap<>();
 
   private long relaysLastValidAfterMillis = -1L;
 
@@ -105,7 +103,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     this.reverseDomainNameResolver = reverseDomainNameResolver;
     this.lookupService = lookupService;
     this.documentStore = DocumentStoreFactory.getDocumentStore();
-    this.now = TimeFactory.getTime().currentTimeMillis();
+    this.now = System.currentTimeMillis();
     this.registerDescriptorListeners();
   }
 
@@ -126,8 +124,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
   /* Step 1: parse descriptors. */
 
-  private SortedSet<String> updatedNodes = new TreeSet<String>();
-
   @Override
   public void processDescriptor(Descriptor descriptor, boolean relay) {
     if (descriptor instanceof ServerDescriptor && relay) {
@@ -147,8 +143,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     }
   }
 
-  private Map<String, SortedSet<String>> declaredFamilies =
-      new HashMap<String, SortedSet<String>>();
+  private Map<String, SortedSet<String>> declaredFamilies = new HashMap<>();
 
   private void processRelayServerDescriptor(
       ServerDescriptor descriptor) {
@@ -163,15 +158,13 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       /* Already parsed more recent server descriptor from this relay. */
       return;
     }
-    long lastRestartedMillis = descriptor.getPublishedMillis()
-        - descriptor.getUptime() * DateTimeHelper.ONE_SECOND;
     int bandwidthRate = descriptor.getBandwidthRate();
     int bandwidthBurst = descriptor.getBandwidthBurst();
     int observedBandwidth = descriptor.getBandwidthObserved();
     int advertisedBandwidth = Math.min(bandwidthRate,
         Math.min(bandwidthBurst, observedBandwidth));
     detailsStatus.setDescPublished(descriptor.getPublishedMillis());
-    detailsStatus.setLastRestarted(lastRestartedMillis);
+    detailsStatus.setLastRestarted(calculateLastRestartedMillis(descriptor));
     detailsStatus.setBandwidthRate(bandwidthRate);
     detailsStatus.setBandwidthBurst(bandwidthBurst);
     detailsStatus.setObservedBandwidth(observedBandwidth);
@@ -180,7 +173,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     detailsStatus.setContact(descriptor.getContact());
     detailsStatus.setPlatform(descriptor.getPlatform());
     if (descriptor.getFamilyEntries() != null) {
-      SortedSet<String> declaredFamily = new TreeSet<String>();
+      SortedSet<String> declaredFamily = new TreeSet<>();
       for (String familyMember : descriptor.getFamilyEntries()) {
         if (familyMember.startsWith("$") && familyMember.length() >= 41) {
           declaredFamily.add(
@@ -195,21 +188,30 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         && (descriptor.getIpv6DefaultPolicy().equals("accept")
         || descriptor.getIpv6DefaultPolicy().equals("reject"))
         && descriptor.getIpv6PortList() != null) {
-      Map<String, List<String>> exitPolicyV6Summary =
-          new HashMap<String, List<String>>();
+      Map<String, List<String>> exitPolicyV6Summary = new HashMap<>();
       List<String> portsOrPortRanges = Arrays.asList(
           descriptor.getIpv6PortList().split(","));
       exitPolicyV6Summary.put(descriptor.getIpv6DefaultPolicy(),
           portsOrPortRanges);
       detailsStatus.setExitPolicyV6Summary(exitPolicyV6Summary);
+    } else {
+      detailsStatus.setExitPolicyV6Summary(null);
     }
     detailsStatus.setHibernating(descriptor.isHibernating() ? true :
         null);
     this.documentStore.store(detailsStatus, fingerprint);
   }
 
-  private Map<String, Map<String, Long>> exitListEntries =
-      new HashMap<String, Map<String, Long>>();
+  private Long calculateLastRestartedMillis(ServerDescriptor descriptor) {
+    Long lastRestartedMillis = null;
+    if (null != descriptor.getUptime()) {
+      lastRestartedMillis = descriptor.getPublishedMillis()
+          - descriptor.getUptime() * DateTimeHelper.ONE_SECOND;
+    }
+    return lastRestartedMillis;
+  }
+
+  private Map<String, Map<String, Long>> exitListEntries = new HashMap<>();
 
   private void processExitList(ExitList exitList) {
     for (ExitList.Entry exitListEntry : exitList.getEntries()) {
@@ -221,8 +223,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           continue;
         }
         if (!this.exitListEntries.containsKey(fingerprint)) {
-          this.exitListEntries.put(fingerprint,
-              new HashMap<String, Long>());
+          this.exitListEntries.put(fingerprint, new HashMap<String, Long>());
         }
         String exitAddress = exitAddressScanMillis.getKey();
         if (!this.exitListEntries.get(fingerprint).containsKey(
@@ -236,11 +237,9 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     }
   }
 
-  private Map<String, Long> lastSeenUnmeasured =
-      new HashMap<String, Long>();
+  private Map<String, Long> lastSeenUnmeasured = new HashMap<>();
 
-  private Map<String, Long> lastSeenMeasured =
-      new HashMap<String, Long>();
+  private Map<String, Long> lastSeenMeasured = new HashMap<>();
 
   private void processRelayNetworkStatusConsensus(
       RelayNetworkStatusConsensus consensus) {
@@ -250,7 +249,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     }
     Set<String> recommendedVersions = null;
     if (consensus.getRecommendedServerVersions() != null) {
-      recommendedVersions = new HashSet<String>();
+      recommendedVersions = new HashSet<>();
       for (String recommendedVersion :
           consensus.getRecommendedServerVersions()) {
         recommendedVersions.add("Tor " + recommendedVersion);
@@ -268,7 +267,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       String address = entry.getAddress();
       int orPort = entry.getOrPort();
       int dirPort = entry.getDirPort();
-      SortedSet<String> orAddressesAndPorts = new TreeSet<String>(
+      SortedSet<String> orAddressesAndPorts = new TreeSet<>(
           entry.getOrAddresses());
       nodeStatus.addLastAddresses(validAfterMillis, address, orPort,
           dirPort, orAddressesAndPorts);
@@ -291,6 +290,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         nodeStatus.setRecommendedVersion((recommendedVersions == null
             || entry.getVersion() == null) ? null :
             recommendedVersions.contains(entry.getVersion()));
+        nodeStatus.setVersion(entry.getVersion());
       }
       if (entry.getUnmeasured()) {
         if (!this.lastSeenUnmeasured.containsKey(fingerprint)
@@ -325,17 +325,14 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       /* Already parsed more recent server descriptor from this bridge. */
       return;
     }
-    long lastRestartedMillis = descriptor.getPublishedMillis()
-        - descriptor.getUptime() * DateTimeHelper.ONE_SECOND;
     int advertisedBandwidth = Math.min(descriptor.getBandwidthRate(),
         Math.min(descriptor.getBandwidthBurst(),
         descriptor.getBandwidthObserved()));
     detailsStatus.setDescPublished(descriptor.getPublishedMillis());
-    detailsStatus.setLastRestarted(lastRestartedMillis);
+    detailsStatus.setLastRestarted(calculateLastRestartedMillis(descriptor));
     detailsStatus.setAdvertisedBandwidth(advertisedBandwidth);
     detailsStatus.setPlatform(descriptor.getPlatform());
     this.documentStore.store(detailsStatus, fingerprint);
-    this.updatedNodes.add(fingerprint);
   }
 
   private void processBridgeExtraInfoDescriptor(
@@ -352,7 +349,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           descriptor.getPublishedMillis());
       detailsStatus.setTransports(descriptor.getTransports());
       this.documentStore.store(detailsStatus, fingerprint);
-      this.updatedNodes.add(fingerprint);
     }
   }
 
@@ -378,7 +374,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         nodeStatus.setRelay(false);
         nodeStatus.setNickname(entry.getNickname());
         nodeStatus.setAddress(entry.getAddress());
-        nodeStatus.setOrAddressesAndPorts(new TreeSet<String>(
+        nodeStatus.setOrAddressesAndPorts(new TreeSet<>(
             entry.getOrAddresses()));
         nodeStatus.setOrPort(entry.getOrPort());
         nodeStatus.setDirPort(entry.getDirPort());
@@ -409,9 +405,9 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
   /* Step 2: read node statuses from disk. */
 
-  private SortedSet<String> currentRelays = new TreeSet<String>();
+  private SortedSet<String> currentRelays = new TreeSet<>();
 
-  private SortedSet<String> runningRelays = new TreeSet<String>();
+  private SortedSet<String> runningRelays = new TreeSet<>();
 
   private void readNodeStatuses() {
     SortedSet<String> previouslyKnownNodes = this.documentStore.list(
@@ -446,7 +442,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     for (Map.Entry<String, NodeStatus> e : this.knownNodes.entrySet()) {
       String fingerprint = e.getKey();
       NodeStatus nodeStatus = e.getValue();
-      this.updatedNodes.add(fingerprint);
       if (nodeStatus.isRelay()
           && nodeStatus.getLastSeenMillis() >= cutoff) {
         this.currentRelays.add(fingerprint);
@@ -495,9 +490,11 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           updatedNodeStatus.setAsNumber(nodeStatus.getAsNumber());
           updatedNodeStatus.setRecommendedVersion(
               nodeStatus.getRecommendedVersion());
+          updatedNodeStatus.setVersion(nodeStatus.getVersion());
         }
         if (nodeStatus.getFirstSeenMillis()
-            < updatedNodeStatus.getFirstSeenMillis()) {
+            < updatedNodeStatus.getFirstSeenMillis()
+            && nodeStatus.getFirstSeenMillis() > 0L) {
           updatedNodeStatus.setFirstSeenMillis(
               nodeStatus.getFirstSeenMillis());
         }
@@ -510,15 +507,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       } else {
         updatedNodeStatus = nodeStatus;
         this.knownNodes.put(fingerprint, nodeStatus);
-        if (nodeStatus.getLastSeenMillis() == (nodeStatus.isRelay()
-            ? previousRelaysLastValidAfterMillis
-            : previousBridgesLastValidAfterMillis)) {
-          /* This relay or bridge was previously running, but we didn't
-           * parse any descriptors with its fingerprint.  Make sure to
-           * update its details status file later on, so it has the
-           * correct running bit. */
-          this.updatedNodes.add(fingerprint);
-        }
       }
       if (updatedNodeStatus.isRelay()
           && updatedNodeStatus.getLastSeenMillis() >= cutoff) {
@@ -548,8 +536,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
    * probabilities. */
 
   private void startReverseDomainNameLookups() {
-    Map<String, Long> addressLastLookupTimes =
-        new HashMap<String, Long>();
+    Map<String, Long> addressLastLookupTimes = new HashMap<>();
     for (String fingerprint : this.currentRelays) {
       NodeStatus nodeStatus = this.knownNodes.get(fingerprint);
       addressLastLookupTimes.put(nodeStatus.getAddress(),
@@ -559,11 +546,10 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     this.reverseDomainNameResolver.startReverseDomainNameLookups();
   }
 
-  private SortedMap<String, LookupResult> geoIpLookupResults =
-      new TreeMap<String, LookupResult>();
+  private SortedMap<String, LookupResult> geoIpLookupResults = new TreeMap<>();
 
   private void lookUpCitiesAndASes() {
-    SortedSet<String> addressStrings = new TreeSet<String>();
+    SortedSet<String> addressStrings = new TreeSet<>();
     for (String fingerprint : this.currentRelays) {
       NodeStatus nodeStatus = this.knownNodes.get(fingerprint);
       addressStrings.add(nodeStatus.getAddress());
@@ -581,22 +567,17 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           nodeStatus.getAddress());
       if (lookupResult != null) {
         this.geoIpLookupResults.put(fingerprint, lookupResult);
-        this.updatedNodes.add(fingerprint);
       }
     }
   }
 
-  private SortedMap<String, Float> consensusWeightFractions =
-      new TreeMap<String, Float>();
+  private SortedMap<String, Float> consensusWeightFractions = new TreeMap<>();
 
-  private SortedMap<String, Float> guardProbabilities =
-      new TreeMap<String, Float>();
+  private SortedMap<String, Float> guardProbabilities = new TreeMap<>();
 
-  private SortedMap<String, Float> middleProbabilities =
-      new TreeMap<String, Float>();
+  private SortedMap<String, Float> middleProbabilities = new TreeMap<>();
 
-  private SortedMap<String, Float> exitProbabilities =
-      new TreeMap<String, Float>();
+  private SortedMap<String, Float> exitProbabilities = new TreeMap<>();
 
   private void calculatePathSelectionProbabilities() {
     boolean consensusContainsBandwidthWeights = false;
@@ -609,7 +590,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     double wee = 0.0;
     double wed = 0.0;
     if (this.lastBandwidthWeights != null) {
-      SortedSet<String> weightKeys = new TreeSet<String>(Arrays.asList(
+      SortedSet<String> weightKeys = new TreeSet<>(Arrays.asList(
           "Wgg,Wgd,Wmg,Wmm,Wme,Wmd,Wee,Wed".split(",")));
       weightKeys.removeAll(this.lastBandwidthWeights.keySet());
       if (weightKeys.isEmpty()) {
@@ -630,14 +611,10 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
           + "this execution.");
       return;
     }
-    SortedMap<String, Double> consensusWeights =
-        new TreeMap<String, Double>();
-    SortedMap<String, Double> guardWeights =
-        new TreeMap<String, Double>();
-    SortedMap<String, Double> middleWeights =
-        new TreeMap<String, Double>();
-    SortedMap<String, Double> exitWeights =
-        new TreeMap<String, Double>();
+    SortedMap<String, Double> consensusWeights = new TreeMap<>();
+    SortedMap<String, Double> guardWeights = new TreeMap<>();
+    SortedMap<String, Double> middleWeights = new TreeMap<>();
+    SortedMap<String, Double> exitWeights = new TreeMap<>();
     double totalConsensusWeight = 0.0;
     double totalGuardWeight = 0.0;
     double totalMiddleWeight = 0.0;
@@ -683,29 +660,24 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       if (consensusWeights.containsKey(fingerprint)) {
         this.consensusWeightFractions.put(fingerprint, (float)
             (consensusWeights.get(fingerprint) / totalConsensusWeight));
-        this.updatedNodes.add(fingerprint);
       }
       if (guardWeights.containsKey(fingerprint)) {
         this.guardProbabilities.put(fingerprint, (float)
             (guardWeights.get(fingerprint) / totalGuardWeight));
-        this.updatedNodes.add(fingerprint);
       }
       if (middleWeights.containsKey(fingerprint)) {
         this.middleProbabilities.put(fingerprint, (float)
             (middleWeights.get(fingerprint) / totalMiddleWeight));
-        this.updatedNodes.add(fingerprint);
       }
       if (exitWeights.containsKey(fingerprint)) {
         this.exitProbabilities.put(fingerprint, (float)
             (exitWeights.get(fingerprint) / totalExitWeight));
-        this.updatedNodes.add(fingerprint);
       }
     }
   }
 
   private void computeEffectiveAndExtendedFamilies() {
-    SortedMap<String, SortedSet<String>> declaredFamilies =
-        new TreeMap<String, SortedSet<String>>();
+    SortedMap<String, SortedSet<String>> declaredFamilies = new TreeMap<>();
     for (String fingerprint : this.currentRelays) {
       NodeStatus nodeStatus = this.knownNodes.get(fingerprint);
       if (nodeStatus != null && nodeStatus.getDeclaredFamily() != null
@@ -713,13 +685,12 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         declaredFamilies.put(fingerprint, nodeStatus.getDeclaredFamily());
       }
     }
-    SortedMap<String, SortedSet<String>> effectiveFamilies =
-        new TreeMap<String, SortedSet<String>>();
+    SortedMap<String, SortedSet<String>> effectiveFamilies = new TreeMap<>();
     for (Map.Entry<String, SortedSet<String>> e :
         declaredFamilies.entrySet()) {
       String fingerprint = e.getKey();
       SortedSet<String> declaredFamily = e.getValue();
-      SortedSet<String> effectiveFamily = new TreeSet<String>();
+      SortedSet<String> effectiveFamily = new TreeSet<>();
       for (String declaredFamilyMember : declaredFamily) {
         if (declaredFamilies.containsKey(declaredFamilyMember)
             && declaredFamilies.get(declaredFamilyMember).contains(
@@ -731,16 +702,15 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
         effectiveFamilies.put(fingerprint, effectiveFamily);
       }
     }
-    SortedMap<String, SortedSet<String>> extendedFamilies =
-        new TreeMap<String, SortedSet<String>>();
-    SortedSet<String> visited = new TreeSet<String>();
+    SortedMap<String, SortedSet<String>> extendedFamilies = new TreeMap<>();
+    SortedSet<String> visited = new TreeSet<>();
     for (String fingerprint : effectiveFamilies.keySet()) {
       if (visited.contains(fingerprint)) {
         continue;
       }
-      SortedSet<String> toVisit = new TreeSet<String>();
+      SortedSet<String> toVisit = new TreeSet<>();
       toVisit.add(fingerprint);
-      SortedSet<String> extendedFamily = new TreeSet<String>();
+      SortedSet<String> extendedFamily = new TreeSet<>();
       while (!toVisit.isEmpty()) {
         String visiting = toVisit.first();
         toVisit.remove(visiting);
@@ -758,7 +728,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       if (extendedFamily.size() > 1) {
         for (String member : extendedFamily) {
           SortedSet<String> extendedFamilyWithoutMember =
-              new TreeSet<String>(extendedFamily);
+              new TreeSet<>(extendedFamily);
           extendedFamilyWithoutMember.remove(member);
           extendedFamilies.put(member, extendedFamilyWithoutMember);
         }
@@ -766,29 +736,23 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
     }
     for (String fingerprint : this.currentRelays) {
       NodeStatus nodeStatus = this.knownNodes.get(fingerprint);
-      if (nodeStatus == null) {
-        continue;
-      }
       if (effectiveFamilies.containsKey(fingerprint)
           || extendedFamilies.containsKey(fingerprint)) {
         nodeStatus.setEffectiveFamily(effectiveFamilies.get(fingerprint));
         nodeStatus.setExtendedFamily(extendedFamilies.get(fingerprint));
-        this.updatedNodes.add(fingerprint);
       } else if ((nodeStatus.getEffectiveFamily() != null
           && !nodeStatus.getEffectiveFamily().isEmpty())
           || (nodeStatus.getIndirectFamily() != null
           && !nodeStatus.getIndirectFamily().isEmpty())) {
         nodeStatus.setEffectiveFamily(null);
         nodeStatus.setExtendedFamily(null);
-        this.updatedNodes.add(fingerprint);
       }
     }
   }
 
   private long startedRdnsLookups = -1L;
 
-  private SortedMap<String, String> rdnsLookupResults =
-      new TreeMap<String, String>();
+  private SortedMap<String, String> rdnsLookupResults = new TreeMap<>();
 
   private void finishReverseDomainNameLookups() {
     this.reverseDomainNameResolver.finishReverseDomainNameLookups();
@@ -801,7 +765,6 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       String hostName = lookupResults.get(nodeStatus.getAddress());
       if (hostName != null) {
         this.rdnsLookupResults.put(fingerprint, hostName);
-        this.updatedNodes.add(fingerprint);
       }
     }
   }
@@ -809,11 +772,9 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
   /* Step 4: update details statuses and then node statuses. */
 
   private void updateNodeDetailsStatuses() {
-    for (String fingerprint : this.updatedNodes) {
-      NodeStatus nodeStatus = this.knownNodes.get(fingerprint);
-      if (nodeStatus == null) {
-        nodeStatus = new NodeStatus(fingerprint);
-      }
+    for (Map.Entry<String, NodeStatus> entry : this.knownNodes.entrySet()) {
+      String fingerprint = entry.getKey();
+      NodeStatus nodeStatus = entry.getValue();
       DetailsStatus detailsStatus = this.documentStore.retrieve(
           DetailsStatus.class, true, fingerprint);
       if (detailsStatus == null) {
@@ -822,7 +783,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
 
       nodeStatus.setContact(detailsStatus.getContact());
 
-      Map<String, Long> exitAddresses = new HashMap<String, Long>();
+      Map<String, Long> exitAddresses = new HashMap<>();
       if (detailsStatus.getExitAddresses() != null) {
         for (Map.Entry<String, Long> e :
             detailsStatus.getExitAddresses().entrySet()) {
@@ -844,7 +805,7 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       }
       detailsStatus.setExitAddresses(exitAddresses);
       SortedSet<String> exitAddressesWithoutOrAddresses =
-          new TreeSet<String>(exitAddresses.keySet());
+          new TreeSet<>(exitAddresses.keySet());
       exitAddressesWithoutOrAddresses.removeAll(
           nodeStatus.getOrAddresses());
       nodeStatus.setExitAddresses(exitAddressesWithoutOrAddresses);
@@ -907,7 +868,8 @@ public class NodeDetailsStatusUpdater implements DescriptorListener,
       }
 
       detailsStatus.setRelay(nodeStatus.isRelay());
-      detailsStatus.setRunning(nodeStatus.getLastSeenMillis()
+      detailsStatus.setRunning(nodeStatus.getRelayFlags().contains("Running")
+          && nodeStatus.getLastSeenMillis()
           == (nodeStatus.isRelay() ? this.relaysLastValidAfterMillis
           : this.bridgesLastPublishedMillis));
       detailsStatus.setNickname(nodeStatus.getNickname());
