@@ -7,6 +7,7 @@ import org.torproject.descriptor.NetworkStatusEntry;
 
 import java.util.List;
 import java.util.Set;
+import java.util.SortedMap;
 import java.util.SortedSet;
 import java.util.TreeSet;
 
@@ -55,6 +56,10 @@ public class DummyStatusEntry implements NetworkStatusEntry {
     return null;
   }
 
+  public Set<String> getMicrodescriptorDigestsSha256Base64() {
+    return null;
+  }
+
   public List<String> getOrAddresses() {
     return null;
   }
@@ -97,5 +102,9 @@ public class DummyStatusEntry implements NetworkStatusEntry {
   public String getMasterKeyEd25519() {
     return null;
   }
+
+  public SortedMap<String, SortedSet<Long>> getProtocols() {
+    return null;
+  }
 }
 
